package lbms.plugins.mldht.azureus.gui;

import lbms.plugins.mldht.azureus.MlDHTPlugin;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;

/**
 * @author Damokles
 *
 */
public class RoutingTableView implements UISWTViewEventListener {

	public static final String	VIEWID			= "mldht_RoutingView";
	private static final int	UPDATE_INTERVAL	= 1000;

	private MlDHTPlugin			plugin;
	private boolean				isCreated		= false;
	private boolean				isActivated		= false;

	private RoutingTableCanvas	rtc;
	private Display				display;

	private Runnable			updateCanvas;

	public RoutingTableView (MlDHTPlugin plugin) {
		this.plugin = plugin;
		updateCanvas = new SWTSafeRunnable() {
			/* (non-Javadoc)
			 * @see lbms.plugins.mldht.azureus.gui.SWTSafeRunnable#runSafe()
			 */
			@Override
			public void runSafe () {
				if (!isActivated) {
					return;
				}
				rtc.fullRepaint();
				if (isActivated && display != null && !display.isDisposed()) {
					display.timerExec(UPDATE_INTERVAL, updateCanvas);
				}
			}
		};
	}

	/**
	 *
	 */
	private void delete () {
		deactivate();
	}

	/**
	 * @param data
	 */
	private void initialize (Composite data) {
		display = data.getDisplay();
		rtc = new RoutingTableCanvas(data);
	}

	/**
	 *
	 */
	private void deactivate () {
		if (!isActivated) {
			return;
		}
		//plugin.getDHT().removeStatsListener(dhtStatsListener);
		rtc.setBucketList(null);
		isActivated = false;
	}

	/**
	 *
	 */
	private void activate () {
		if (!isCreated || isActivated) {
			return;
		}
		rtc.setBucketList(plugin.getDHT().getNode().getBuckets());
		if (display != null && !display.isDisposed()) {
			display.timerExec(UPDATE_INTERVAL, updateCanvas);
		}

		isActivated = true;
	}

	/* (non-Javadoc)
	 * @see org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener#eventOccurred(org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent)
	 */
	public boolean eventOccurred (UISWTViewEvent event) {
		switch (event.getType()) {

		case UISWTViewEvent.TYPE_CREATE:
			if (isCreated) {
				return false;
			}
			isCreated = true;
			break;

		case UISWTViewEvent.TYPE_FOCUSGAINED:
			activate();
			break;

		case UISWTViewEvent.TYPE_FOCUSLOST:
			deactivate();
			break;

		case UISWTViewEvent.TYPE_INITIALIZE:
			initialize((Composite) event.getData());
			break;

		case UISWTViewEvent.TYPE_CLOSE:
		case UISWTViewEvent.TYPE_DESTROY:
			delete();
			break;
		}
		return true;
	}
}
