/**
 * 
 */
package lbms.plugins.mldht.azureus.gui;

import java.text.DecimalFormat;

import lbms.plugins.mldht.azureus.MlDHTPlugin;
import lbms.plugins.mldht.kad.DHTStatus;
import lbms.plugins.mldht.kad.DHTStatusListener;
import lbms.plugins.mldht.kad.PopulationEstimator;
import lbms.plugins.mldht.kad.PopulationListener;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.ui.swt.plugins.UISWTStatusEntry;

/**
 * @author Leonard
 * 
 */
public class SWTHelper implements UIManagerListener {
	private UISWTStatusEntry	statusEntry;
	private DHTStatusListener	dhtStatusListener;
	private PopulationListener	populationListener;
	private UISWTInstance		swtInstance;
	private MlDHTPlugin			plugin;
	private DHTView				view;

	public Image				dhtStatusEntryIcon;
	public Display				display;

	public SWTHelper(MlDHTPlugin plugin) {
		this.plugin = plugin;
	}

	public void onPluginUnload () {
		PopulationEstimator.removeListener(populationListener);
		if (statusEntry != null) {
			statusEntry.destroy();
			statusEntry = null;
		}
		if (dhtStatusEntryIcon != null) {
			dhtStatusEntryIcon.dispose();
			dhtStatusEntryIcon = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.gudy.azureus2.plugins.ui.UIManagerListener#UIAttached(org.gudy.azureus2.plugins.ui.UIInstance)
	 */
	public void UIAttached (UIInstance instance) {

		if (instance instanceof UISWTInstance) {
			swtInstance = (UISWTInstance) instance;
			display = swtInstance.getDisplay();

			if (view == null) {
				view = new DHTView(plugin, swtInstance.getDisplay());
			}
			swtInstance.addView(UISWTInstance.VIEW_MAIN, DHTView.VIEWID, view);
			try {
				statusEntry = swtInstance.createStatusEntry();
				dhtStatusEntryIcon = new Image(
						swtInstance.getDisplay(),
						MlDHTPlugin.class
								.getResourceAsStream("/lbms/plugins/mldht/azureus/gui/dhtIcon.png"));
				statusEntry.setImage(dhtStatusEntryIcon);
				statusEntry.setImageEnabled(true);
			} catch (RuntimeException e) {
				e.printStackTrace();
			}

			if (plugin.isPluginAutoOpen()) {
				swtInstance.openMainView(DHTView.VIEWID, view, null);
			}
			populationListener = new PopulationListener() {
				private DecimalFormat	format	= new DecimalFormat();

				/*
				 * (non-Javadoc)
				 * 
				 * @see lbms.plugins.mldht.kad.PopulationListener#populationUpdated(long)
				 */
				public void populationUpdated (long estimatedPopulation) {

					if (statusEntry != null
							&& PopulationEstimator.getEstimate() != 0) {
						if (display != null && !display.isDisposed()) {
							display.asyncExec(new Runnable() {
								public void run () {
									statusEntry.setText("mlDHT: "
											+ format.format(PopulationEstimator
													.getEstimate()));
								}
							});
						}
					}

				}
			};

			PopulationEstimator.addListener(populationListener);

			dhtStatusListener = new DHTStatusListener() {
				/*
				 * (non-Javadoc)
				 * 
				 * @see lbms.plugins.mldht.kad.DHTStatusListener#statusChanged(lbms.plugins.mldht.kad.DHTStatus,
				 *      lbms.plugins.mldht.kad.DHTStatus)
				 */
				public void statusChanged (DHTStatus newStatus,
						DHTStatus oldStatus) {
					if (statusEntry != null) {
						switch (newStatus) {
						case Running:
							break;

						default:
							statusEntry.setText("mlDHT: " + newStatus);
							break;
						}

						statusEntry.setTooltipText(newStatus.toString());
						statusEntry.setVisible(plugin.getPluginInterface()
								.getPluginconfig().getPluginBooleanParameter(
										"showStatusEntry"));
					}
				}
			};

			dhtStatusListener.statusChanged(plugin.getDHT().getStatus(),
					DHTStatus.Stopped);

			plugin.getDHT().addStatusListener(dhtStatusListener);
		}
	}

	public void UIDetached (UIInstance instance) {
		if (instance instanceof UISWTInstance) {
			swtInstance = null;
			display = null;
		}
	}
}
