package lbms.plugins.mldht.kad;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.util.Arrays;
import java.util.Comparator;

/**
 * @author Damokles
 *
 */
public class DBItem {
	private final byte[]	item	= new byte[6];
	private final long	time_stamp;

	public DBItem () {
		time_stamp = System.currentTimeMillis();
	}

	public DBItem (final byte[] ip_port) {
		this();
		System.arraycopy(ip_port, 0, item, 0, item.length);
	}

	public DBItem (final byte[] buffer, final int offset) {
		this();
		System.arraycopy(buffer, offset, item, 0, item.length);
	}

	public DBItem (final DBItem dbItem) {
		this();
		System.arraycopy(dbItem.item, 0, item, 0, item.length);
	}

	/// See if the item is expired
	public boolean expired (final long now) {
		return (now - time_stamp >= DHTConstants.MAX_ITEM_AGE);
	}

	/// Get the data of an item
	public byte[] getData () {
		return item;
	}

	@Override
	public String toString() {
		try
		{
			return new InetSocketAddress(InetAddress.getByAddress(Arrays.copyOf(item, 4)),(item[4]&0xff)<<8 | item[5]&0xff).toString();
		} catch (final Exception e)
		{
			return super.toString();
		}
	}

	@Override
	public boolean equals(final Object obj) {
		if(obj instanceof DBItem)
		{
			byte[] otherItem = ((DBItem)obj).item;
			for(int i=0;i<4;i++)
				if(item[i] != otherItem[i])
					return false;
			return true;
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return Arrays.hashCode(item);
	}

	public static final Comparator<DBItem> ageOrdering = new Comparator<DBItem>() {
		public int compare(final DBItem o1, final DBItem o2) {
			return (int)(o1.time_stamp - o2.time_stamp);
		}
	};
}
