package lbms.plugins.mldht.kad.messages;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import lbms.plugins.mldht.kad.DHT;
import lbms.plugins.mldht.kad.DHTConstants;
import lbms.plugins.mldht.kad.Key;

import org.gudy.azureus2.core3.util.BEncoder;

/**
 * @author Damokles
 *
 */
public class FindNodeRequest extends MessageBase {

	private Key	target;

	/**
	 * @param id
	 * @param target
	 */
	public FindNodeRequest (Key id, Key target) {
		super(new byte[] { (byte)0xff }, Method.FIND_NODE, Type.REQ_MSG, id);
		this.target = target;
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#apply(lbms.plugins.mldht.kad.DHT)
	 */
	@Override
	public void apply (DHT dh_table) {
		dh_table.findNode(this);
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#encode()
	 */
	@Override
	public byte[] encode () throws IOException {
		Map<String, Object> base = new HashMap<String, Object>();
		Map<String, Object> inner = new HashMap<String, Object>();
		base.put(DHTConstants.ARG, inner);
		inner.put("id", id.getHash());
		inner.put("target", target.getHash());

		base.put(DHTConstants.REQ, "find_node");
		base.put(DHTConstants.TID, mtid);
		base.put(DHTConstants.TYP, DHTConstants.REQ);
		base.put(DHTConstants.VER, DHTConstants.getVersion());

		return BEncoder.encode(base);
	}

	/**
	 * @return the target
	 */
	public Key getTarget () {
		return target;
	}
	
	@Override
	public String toString() {
		return super.toString() + "targetKey:"+target+" ("+(160-DHT.getSingleton().getOurID().findApproxKeyDistance(target))+")";
	}
}
