package lbms.plugins.mldht.kad.messages;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import lbms.plugins.mldht.kad.DHT;
import lbms.plugins.mldht.kad.DHTConstants;
import lbms.plugins.mldht.kad.Key;

import org.gudy.azureus2.core3.util.BEncoder;

/**
 * @author Damokles
 *
 */
public class GetPeersRequest extends MessageBase {

	protected Key	info_hash;

	/**
	 * @param id
	 * @param info_hash
	 */
	public GetPeersRequest (Key id, Key info_hash) {
		super(new byte[] {(byte) 0xFF}, Method.GET_PEERS, Type.REQ_MSG, id);
		this.info_hash = info_hash;
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#apply(lbms.plugins.mldht.kad.DHT)
	 */
	@Override
	public void apply (DHT dh_table) {
		dh_table.getPeers(this);
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#encode()
	 */
	@Override
	public byte[] encode () throws IOException {
		Map<String, Object> base = new HashMap<String, Object>();
		Map<String, Object> inner = new HashMap<String, Object>();
		base.put(DHTConstants.ARG, inner);
		inner.put("id", id.getHash());
		inner.put("info_hash", info_hash.getHash());

		base.put(DHTConstants.REQ, "get_peers");
		base.put(DHTConstants.TID, mtid);
		base.put(DHTConstants.TYP, DHTConstants.REQ);
		base.put(DHTConstants.VER, DHTConstants.getVersion());

		return BEncoder.encode(base);
	}

	/**
	 * @return the info_hash
	 */
	public Key getInfoHash () {
		return info_hash;
	}
	
	public String toString() {
		return super.toString() + "targetKey:"+info_hash+" ("+(160-DHT.getSingleton().getOurID().findApproxKeyDistance(info_hash))+")";
	}
}
